
import numpy as np
from becalib.layers import MaterialLayer
from becalib.translator import get_translator


# Dynamic Thermal Analysis #
def get_periodic_penetration_depth_list(
        layers: list[MaterialLayer],
        time_period:float=24
        ) -> np.ndarray:   
    """ list of periodic penetration depth δ in [m] layer by layer
    Args:
        layers (list[MaterialLayer]): ordered list of material layers interior to exterior
        time_period (float, optional): analysis period in [h]. Defaults to 24 h.

    Returns:
        np.ndarray: periodic penetration depths
    """
    
    periodic_penetration_depth_list= []
    time_in_seconds= time_period*3600

    for layer in layers:
        if layer.is_air is False:
            delta_layer= np.sqrt(
                (layer.thermal_conductivity*time_in_seconds)
                / (np.pi * layer.gross_density * layer.specific_heat_capacity)
                )
            periodic_penetration_depth_list.append(delta_layer)
        else:
            periodic_penetration_depth_list.append(None)

    
    return np.array(periodic_penetration_depth_list)


def get_xi_list(
    layers: list[MaterialLayer],
    periodic_penetration_depth_list:np.ndarray,
        ) -> np.ndarray:
    """list of ξ values = s/d thickness / periodic_penetration
        Dimensionless quantity
    Args:
        layers (list[MaterialLayer]): ordered list of material layers interior to exterior
        list of periodic penetration depth δ in [m] layer by layer

    Returns:
        np.ndarray:  ξ = s/d
    """

    xi_list=[]

    i=0

    for layer in layers:
        if layer.is_air is False:
            xi_i= layer.thickness / periodic_penetration_depth_list[i]
            xi_list.append(xi_i)
        else:
            xi_list.append(None)
        
        i +=1

    return np.array(xi_list)


def get_heat_transfer_matrix_layer_list(thermal_resistances:np.ndarray,
                                        xi_list:np.ndarray,
                                        periodic_penetration_depth_list:np.ndarray,
                                        thermal_conductivities:np.ndarray,
                                        ) -> list[np.ndarray]:
    """list of heat transfer matrix layer by layer

    Args:
        thermal_resistances (list[float]):array of all thermal resistances Layer by layer
        xi_list(list): list of ξ values = s/d
        periodic_penetration_depth_list(list):periodic penetration depth δ in [m] layer by layer
        thermal_conductivities(list): thermal conductivities "λ" lambda [W/mK] Layer by Layer 

    Returns:
        list[np.ndarray]: list of heat transfer matrix 
    """
        
    resistances= np.delete(thermal_resistances, 0)
    pp_depths = periodic_penetration_depth_list
    conduct_ies = thermal_conductivities

    ht_matrix_list: list[np.ndarray] = []  # zz : z matrix list
    for i in range(0, len(xi_list)):
        # z is a layer matrix 2X2 of complex values
        z = np.zeros((2, 2), dtype=np.complex128) 
    
        if   xi_list[i] is None:
            # air_layer
            z[0][0] = 1
            z[1][1] = z[0][0]
            z[0][1]= -resistances[i]
            z[1][0]=0

        else: 
            # material layer
            z[0][0] = complex(
                (np.cosh(xi_list[i]) * np.cos(xi_list[i])), 
                (np.sinh(xi_list[i]) * np.sin(xi_list[i]))
                )
            z[1][1] = z[0][0]

            z[0][1] = -(pp_depths[i] / (2 * conduct_ies[i])) * \
                complex(
                    (np.sinh(xi_list[i]) * np.cos(xi_list[i]) + np.cosh(xi_list[i]) * np.sin(xi_list[i])),
                    (np.cosh(xi_list[i]) * np.sin(xi_list[i]) - np.sinh(xi_list[i]) * np.cos(xi_list[i])),
                    )
            z[1][0] = -(conduct_ies[i] / (pp_depths[i])) * \
                complex(
                    (np.sinh(xi_list[i]) * np.cos(xi_list[i]) - np.cosh(xi_list[i]) * np.sin(xi_list[i])),
                    (np.sinh(xi_list[i]) * np.cos(xi_list[i]) + np.cosh(xi_list[i]) * np.sin(xi_list[i])),
                    )
        # add z to zz list
        ht_matrix_list.append(z)

    return ht_matrix_list


def get_heat_transfer_matrix_component(
        ht_matrix_list:list[np.ndarray],
        surface_thermal_resistance_int:float,
        surface_thermal_resistance_ext:float) -> np.ndarray:
    """heat transfer matrix f multi layers components
        Z = Z_i * Z_1 * Z_1 * Z_2 ... * Z_N-1 * Z_N * Z_e
    
    Args:
        ht_matrix_list(list): list of heat transfer matrix layer by layer
        surface_thermal_resistance_int(float): Internal surface resistances Rsi
        surface_thermal_resistance_ext(float): External surface resistances Rse

    Returns:
        np.ndarray: heat_transfer_matrix [Z]
    """

    htm = np.zeros((2, 2), dtype=np.complex128)
    htm = ht_matrix_list[-1]  # Z_N

    for i in range(1, len(ht_matrix_list)):  
        htm = htm.dot(ht_matrix_list[-1 - i])  # dot product or scalar product


    # heat transfer matrix interior surface layer
    Z_i = np.array(
        [
            [complex(1, 0), complex(-surface_thermal_resistance_int)],
            [complex(0, 0), complex(1, 0)],
        ],
        dtype=np.complex128,
    )

    ## heat transfer matrix exterior surface layer
    Z_e = np.array(
        [
            [complex(1, 0), complex(-surface_thermal_resistance_ext)],
            [complex(0, 0), complex(1, 0)],
        ],
        dtype=np.complex128,
    )

    htm = Z_e.dot(htm)# dot product or scalar product
    htm = htm.dot(Z_i)# dot product or scalar product

    return htm

def get_periodic_thermal_transmittance(heat_transfer_matrix_component) -> float:
    """periodic_thermal_transmittance component value \n
        Yie in W/m²K

    Args:
        heat_transfer_matrix_component(array): heat transfer matrix f multi layers components

    Returns:
        float: Yie in W/m²K
    """

    Z_12  = heat_transfer_matrix_component[0][1]

    mod_Z_12 = np.sqrt((Z_12.real) ** 2 + (Z_12.imag) ** 2) # modulo 

    Y_ie = 1/mod_Z_12

    return Y_ie


def get_decrement_factor(periodic_thermal_transmittance,
                        thermal_transmittance_component
                        ) -> float:
    """decrement_factor = Y_ie / u-value
    Args:
        periodic_thermal_transmittance(float)
        thermal_transmittance_component(float)

    Returns:
        float: decrement_factor [-]
    """
    return (
        periodic_thermal_transmittance / thermal_transmittance_component
    )

def get_time_shift(heat_transfer_matrix_component,
                   time_period) -> float:
    """time_shift

    Args:
        heat_transfer_matrix_component(np.ndarray):
        time_period(float):

    Returns:
        float: time_shift in [h]
    """

    htm = heat_transfer_matrix_component
    htm_12= htm[0][1]

    phase=(np.arctan2(htm_12.imag, htm_12.real)) * time_period / (2 * np.pi)

    return phase + time_period / 2


def get_thermal_admittance_int(heat_transfer_matrix_component) -> float:
    """thermal_admittance_int Y_ii in W/m²K
    Args:
        heat_transfer_matrix_component(np.ndarray):

    Returns:
        float: Y_ii in W/m²K

    """
    htm = heat_transfer_matrix_component

    Y_ii = -htm[0][0] / htm[0][1]
    Y_ii = np.sqrt((Y_ii.real) ** 2 + (Y_ii.imag) ** 2)  # the module

    return Y_ii


def get_thermal_admittance_ext(heat_transfer_matrix_component) -> float:
    """thermal_admittance_int Y_ee in W/m²K
    Args:
        heat_transfer_matrix_component(np.ndarray):

    Returns:
        float: Y_ee in W/m²K

    """

    htm = heat_transfer_matrix_component

    Y_ee = -htm[1][1] / htm[0][1]
    Y_ee = np.sqrt((Y_ee.real) ** 2 + (Y_ee.imag) ** 2)  # the module

    return Y_ee


def get_areal_heat_capacity_int(heat_transfer_matrix_component,
                                time_period:float) -> float:
    """interior areal heat capacity [kJ/m²K]
        k1 = P/ 2 pi |Y_22-Y_12|
    
    Args:
        heat_transfer_matrix_component(np.ndarray):
        time_period(float): in hours

    Returns:
        float: [kJ/m²K]
    """
    
    htm = heat_transfer_matrix_component
    return (
        (time_period * 3600)
        / (2 * np.pi)
        * np.sqrt(
            (((htm[0][0] - 1) / htm[0][1]).real) ** 2
            + (((htm[0][0] - 1) / htm[0][1]).imag) ** 2
        )
    ) / 1000  # kJ/m2 K


def get_areal_heat_capacity_ext(heat_transfer_matrix_component,
                                time_period) -> float:
    """exterior areal heat capacity  [kJ/m²K]
        k2 = P/ 2 pi |Y_11-Y_12|
    Args:
        heat_transfer_matrix_component(np.ndarray):
        time_period(float): in hours

    Returns:
        float: [kJ/m²K]
    """


    htm = heat_transfer_matrix_component
    return (
        (time_period * 3600)
        / (2 * np.pi)
        * np.sqrt(
            (((htm[1][1] - 1) / htm[0][1]).real) ** 2
            + (((htm[1][1] - 1) / htm[0][1]).imag) ** 2
        )
    ) / 1000  


def get_areal_heat_capacity_component(
        layers: list[MaterialLayer]) -> float:
    """component areal heat capacity in [kJ/m²K]
        areal heat capacity of all layers

    Args:
        layers (list[MaterialLayer]): ordered list of material layers interior to exterior

    Returns:
        float: [kJ/m²K]
    """

    areal_heat_capacity_list=[]

    for layer in layers:
        if layer.is_air is False:
            areal_heat_capacity_list.append(layer.gross_density*layer.thickness*layer.specific_heat_capacity)

        else:
            #TODO check if 0 is ok for air layer
            areal_heat_capacity_list.append(0)

    return np.sum(np.array(areal_heat_capacity_list))/1000


def get_time_constant(areal_heat_capacity_component,thermal_resistance_component)->float:
    """time_constant in h
    Args:
        areal_heat_capacity_component(float): component areal heat capacity in [kJ/m²K]
        thermal_resistance_component(float): Component thermal resistance R in [m²K/W]

    Returns:
        float: h
    """
    return areal_heat_capacity_component*thermal_resistance_component*1000/3600


def get_threshold_values_italian_dm_26_06_2009(
        time_shift:float,
        decrement_factor:float,
        language:str="en"):
    """threshold values in accordance with italian rule DM 26/06/2009 \n
        Possible scores: \n
            Excellent 5/5 \n
            Good 4/5 \n
            Medium 3/5 \n
            Sufficient 2/5 \n
            Poor 1/5 \n
            Impossible score

    Args:
        time_shift(float): in hours
        decrement_factor(float): [-]
        language(str): en, fr, etc

    Returns:
        str: score 
    """
    _=get_translator(language=language)

    if time_shift >12 and decrement_factor<0.15:
        return _("Excellent 5/5") 
    
    elif (10<time_shift <=12) and (0.15 <= decrement_factor<0.3):
        return _("Good 4/5")
    
    elif (8<time_shift <=10) and (0.3 <= decrement_factor<0.4):
        return _("Medium 3/5") 
    
    elif (6<time_shift <=8) and (0.4 <= decrement_factor<0.6):
        return _("Sufficient 2/5") 
    
    elif time_shift <=6 and (0.6 <= decrement_factor):
        return _("Poor 1/5") 
    else:
        return _("Impossible score") 
    

def get_mass_component(layers: list[MaterialLayer]) -> float:
    """component mass per square meters in kg/m²
    Args:
        layers (list[MaterialLayer]): ordered list of material layers interior to exterior

    Returns:
        float: component mass per square meters in [kg/m²]
    """
    mass_list = []
    for layer in layers:
        if layer.is_air is False:
            mass_list.append(layer.gross_density*layer.thickness)
        else:
            # if air add 0
            mass_list.append(0)

    return np.sum(np.array(mass_list))








import unittest
from becalib import MaterialLayer, AirLayer
from becalib import Component
import math
import pandas as pd

#plt.style.use(["science", "retro", "no-latex"])


class TestSummerWallPerformance(unittest.TestCase):

    def test_layer(self):
        """check layer computed values  thermal_diffusivity and thermal_effusivity
        """
        plaster = MaterialLayer(
        name="Plaster",
        thickness=0.15, 
        thermal_conductivity=0.21, 
        gross_density=1150, 
        specific_heat_capacity=1100, 
        )

        self.assertTrue(math.isclose(0.166,plaster.thermal_diffusivity,rel_tol=0.01))
        self.assertTrue(math.isclose(515.41,plaster.thermal_effusivity,rel_tol=0.01))
    
    def test_layer_get_values_fr(self):
        """check layer en->fr translation
        """

        plaster = MaterialLayer(
        name="Plaster",
        thickness=0.15, # m
        thermal_conductivity=0.21, # # W/mK W/mK
        gross_density=1150, # ro kg/mc
        specific_heat_capacity=1100, #  c J/kgK ,3,
        language="fr"
        )

        out_str=plaster.get_values()
        self.assertEqual("Épaisseur : 0.150 [m]",out_str.splitlines()[2])

    def test_air_layer_get_values_fr(self):
        """check air layer en->fr translation
        """

        air_gap = AirLayer(
        name="air_gap",
        thickness=0.19, # m
        heat_flow_direction="Up",
        language="fr"
        )

        out_str=air_gap.get_values()
        self.assertEqual("Conductivité thermique λ : 1.188 [W/mK]",out_str.splitlines()[3])


    def test_wall_get_values_fr(self):
        """check Component get_value method en->fr translation
        """       
        plasterboard = MaterialLayer(
            name="PLACOPLATRE",
            thickness=0.015,
            thermal_conductivity=0.21,
            gross_density=1150,
            specific_heat_capacity=1100,
        )
        clt_panel = MaterialLayer(
            name="Panneau CLT",
            thickness=0.096,
            thermal_conductivity=0.13,
            gross_density=500.0,
            specific_heat_capacity=1600,
        )
        air_gap= AirLayer(
            name= "lame d'air", 
            thickness=0.05,
        )
        insulation = MaterialLayer(
            name="isolant haute densité",
            thickness=0.13,
            thermal_conductivity=0.043,
            gross_density=190,
            specific_heat_capacity=2100,
        )
        plaster = MaterialLayer(
            name="Enduit",
            thickness=0.015,
            thermal_conductivity=0.9,
            gross_density=1800,
            specific_heat_capacity=1000,
        )

        wall_3c = Component(name="Wall_3c", 
                            layers=[plasterboard, 
                                    clt_panel, 
                                    insulation, 
                                    air_gap,
                                    plaster],
                            heat_flow_direction="Ho", 
                            language="fr")

        # print(wall_3c.get_values())

        df=wall_3c.get_layers_dataframe()


    def test_wall_values(self):    

        concrete  =  MaterialLayer(
            name="concrete",
            thickness=0.3, # m
            thermal_conductivity=1.8, # W/mK W/mK
            specific_heat_capacity=1000, #  c J/kgK            
            gross_density=2400, # ro kg/mc
        )

        insulation_a  =  MaterialLayer(
            name="insulation_a",
            thickness=0.1, # m
            thermal_conductivity=0.034, # W/mK W/mK
            specific_heat_capacity=700, #  c J/kgK            
            gross_density=70, # ro kg/mc
        )



        brick_a  =  MaterialLayer(
            name="brick_a",
            thickness=0.08, # m
            thermal_conductivity=0.35, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=750, # ro kg/mc
        )

        brick_b  =  MaterialLayer(
            name="brick_b",
            thickness=0.12, # m
            thermal_conductivity=0.8, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=1800, # ro kg/mc
        )

        plaster  =  MaterialLayer(
            name="plaster",
            thickness=0.02, # m
            thermal_conductivity=0.9, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=1400, # ro kg/mc
        )


        wall = Component(name="Wall Test", 
                        layers=[
                        concrete,
                        insulation_a,
                        brick_a,
                        brick_b,
                        plaster ],
                        heat_flow_direction="Ho",
                        language="en"
                        )
        

        approx_rel_tot= 0.001

        self.assertTrue(math.isclose(0.62,wall.thickness_component,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(0.03239,wall.decrement_factor,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(17.854,wall.time_shift,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(0.008804,wall.periodic_thermal_transmittance,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(5.7303,wall.thermal_admittance_int,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(7.7158,wall.thermal_admittance_ext,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(78.7761,wall.areal_heat_capacity_int,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(106.035,wall.areal_heat_capacity_ext,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(1031,wall.mass_component,rel_tol=approx_rel_tot))

        self.assertTrue(math.isclose(3.679,wall.thermal_resistance_component,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(0.272,wall.thermal_transmittance_component,rel_tol=approx_rel_tot))

        self.assertTrue(math.isclose(980,wall.areal_heat_capacity_component,rel_tol=approx_rel_tot))
        self.assertTrue(math.isclose(1002,wall.time_constant,rel_tol=approx_rel_tot))

        self.assertEqual("Excellent 5/5",wall.threshold_values_italian_dm_26_06_2009)

    def test_wall_values_2(self):    

        concrete  =  MaterialLayer(
            name="concrete",
            thickness=0.1, # m
            thermal_conductivity=1.8, # W/mK W/mK
            specific_heat_capacity=1000, #  c J/kgK            
            gross_density=2400, # ro kg/mc
        )

        air  =  AirLayer(
            name="air",
            thickness=0.1, # m
            heat_flow_direction="Do"

        )

        brick_a  =  MaterialLayer(
            name="brick_a",
            thickness=0.08, # m
            thermal_conductivity=0.35, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=750, # ro kg/mc

        )

        brick_b  =  MaterialLayer(
            name="brick_b",
            thickness=0.12, # m
            thermal_conductivity=0.8, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=1800, # ro kg/mc

        )
        iso  =  MaterialLayer(
            name="iso",
            thickness=0.05, # m
            thermal_conductivity=0.035, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=175, # ro kg/mc

        )

        plaster  =  MaterialLayer(
            name="plaster",
            thickness=0.02, # m
            thermal_conductivity=0.9, # W/mK W/mK
            specific_heat_capacity=840, #  c J/kgK            
            gross_density=1400, # ro kg/mc

        )


        wall = Component(name="Wall", 
                        layers=[
                        concrete,
                        air,
                        brick_a,
                        brick_b,
                        iso,
                        plaster ],
                        heat_flow_direction="Ho"

                        )
        

        approx_rel_tot= 0.01

        self.assertTrue(math.isclose(0.47,wall.thickness_component,rel_tol=approx_rel_tot)) # m spessore totale
        self.assertTrue(math.isclose(0.06014,wall.decrement_factor,rel_tol=approx_rel_tot)) # - fd Fattore di decremento (attenuazione)
        self.assertTrue(math.isclose(13.376,wall.time_shift,rel_tol=approx_rel_tot)) #  phi (h) Ritardo fattore di decremento (sfasamento)
        self.assertTrue(math.isclose(0.026908,wall.periodic_thermal_transmittance,rel_tol=approx_rel_tot)) # Yie  # [W/m2K] Trasmittanza termica periodica 
        self.assertTrue(math.isclose(6.0131,wall.thermal_admittance_int,rel_tol=approx_rel_tot)) # Yii # [W/m2K] Ammettenza termica lato interno 
        self.assertTrue(math.isclose(1.9639,wall.thermal_admittance_ext,rel_tol=approx_rel_tot)) #  Yee  # [W/m2K] Ammettenza termica lato esterno 
        self.assertTrue(math.isclose(82.966,wall.areal_heat_capacity_int,rel_tol=approx_rel_tot)) # k1  #[kJ/m2K] Capacità termica areica lato interno 
        self.assertTrue(math.isclose(27.034,wall.areal_heat_capacity_ext,rel_tol=approx_rel_tot)) # k2=  #[kJ/m2K] Capacità termica areica lato esterno 
        self.assertTrue(math.isclose(553,wall.mass_component,rel_tol=approx_rel_tot)) # Ms  # [kg/m2]	Massa superficiale	
        self.assertTrue(math.isclose(2.235,wall.thermal_resistance_component,rel_tol=approx_rel_tot)) #  Rt  # [m2K/W]	Resistenza termica totale 	
        self.assertTrue(math.isclose(0.447,wall.thermal_transmittance_component,rel_tol=approx_rel_tot))  # U   # [W/m2K]	Trasmittanza 	
        self.assertTrue(math.isclose(503,wall.areal_heat_capacity_component,rel_tol=approx_rel_tot)) # Cta  # [kJ/m2K]	Capacità termica areica 	
        self.assertTrue(math.isclose(312,wall.time_constant,rel_tol=approx_rel_tot)) # t  # [h] Costante di tempo	

        self.assertEqual("Excellent 5/5",wall.threshold_values_italian_dm_26_06_2009)

    def test_wall_fr(self):

        beton  =  MaterialLayer(
            name="Béton",
            thickness=0.1, # m
            thermal_conductivity=1.8, # lambda W/mK
            specific_heat_capacity=1000, # calore spec c J/kgK            
            gross_density=2400, # ro kg/mc
        )

        couche_air  =  AirLayer(
            name="Couche d'air",
            thickness=0.1, # m
            heat_flow_direction="Do"

        )

        monomur  =  MaterialLayer(
            name="Monomur",
            thickness=0.08, # m
            thermal_conductivity=0.35, # lambda W/mK
            specific_heat_capacity=840, # calore spec c J/kgK            
            gross_density=750, # ro kg/mc

        )

        brique  =  MaterialLayer(
            name="Brique",
            thickness=0.12, # m
            thermal_conductivity=0.8, # lambda W/mK
            specific_heat_capacity=840, # calore spec c J/kgK            
            gross_density=1800, # ro kg/mc

        )
        isolant  =  MaterialLayer(
            name="Isolant",
            thickness=0.05, # m
            thermal_conductivity=0.035, # lambda W/mK
            specific_heat_capacity=840, # calore spec c J/kgK            
            gross_density=175, # ro kg/mc

        )

        enduit  =  MaterialLayer(
            name="Enduit",
            thickness=0.02, # m
            thermal_conductivity=0.9, # lambda W/mK
            specific_heat_capacity=840, # calore spec c J/kgK            
            gross_density=1400, # ro kg/mc

        )


        paroi = Component(name="Paroi", 
                        layers=[
                        beton,
                        couche_air,
                        monomur,
                        brique,
                        isolant,
                        enduit ],
                        heat_flow_direction="Do",
                        language="fr",
                        time_period=24

                        )
        
        self.assertIsInstance(paroi.get_layers_dataframe(data_type="df"),pd.DataFrame)
        self.assertEqual(type(paroi.get_layers_dataframe(data_type="st")),type(pd.DataFrame().style.format()))
        out_str=paroi.get_values()
        self.assertEqual("Période : 24 [h]",out_str.splitlines()[4])
        out_str=paroi.get_summer_performance_key_values()
        self.assertEqual("Résistance: 2.315 [m²K/W] Rsi and Rse included",out_str.splitlines()[6])



if __name__ == '__main__':
    unittest.main()


import numpy as np
from becalib.air_resistances import get_resistance_unventilated_air_layer
from becalib.translator import get_translator


class LayerBase():
    """common layer parameter of materials and air layers
    """
    def __init__(self, 
        name:str,
        thickness:float, # m
        is_air:bool =False,
        language:str="en"
                 ):
        """input parameters

        Args:
            name (str): name of layer
            thickness (float): "d" in m thickness of layer
        """
        
        self.name = name
        if not type(thickness)  is float:
            raise ValueError("thickness: have to be à float type")
        
        self.thickness= thickness # m
        self.is_air= is_air
        self.language=language


class MaterialLayer(LayerBase):
    """Material Layer 
    Args:
        LayerBase (Class): inheritance of parameters from LayerBase class
    """
    def __init__(self,
        name:str,
        thickness:float, # m
        thermal_conductivity:float, 
        gross_density:float,
        specific_heat_capacity:float,
        is_air:bool =False,
        language:str="en" 

        ):
        
        """Material Layer input parameters

        Args:
            name (str): 
            thickness (float):  "d" in m thickness of layer
            thermal_conductivity (float): "λ" lambda [W/mK]
            gross_density (float): "ρ" rho [kg/mc]
            specific_heat_capacity (float): "c" [J/kgK]
            is_air (bool, optional): Defaults to False
            language (str, optional): Defaults to "en". 
        """

        super().__init__(name,
                      thickness,
                      is_air,
                      language)



        self.thermal_conductivity= thermal_conductivity  # λ [W/mK]
        self.gross_density =gross_density # ro [kg/m³]
        self.specific_heat_capacity= specific_heat_capacity  # c [J/kgK]   
    
    

    @property
    def thermal_resistance(self):
        """R in m²K/W
        Returns:
            float: Thermal resistance d/λ
        """

        return self.thickness / self.thermal_conductivity
    
    
    @property
    def thermal_diffusivity(self):
        """alpha = lambda/(ro * c)  in [m²/ (s*10^6)]
        Returns:
            float: [m²/ (s*10^6)]
        """
        return (self.thermal_conductivity/self.specific_heat_capacity/self.gross_density)*10**6
    

    @property
    def thermal_effusivity(self):
        """thermal_effusivity

        Returns:
            float: thermal_effusivity
        """
        return np.sqrt(self.thermal_conductivity*self.specific_heat_capacity*self.gross_density)



    def get_values(self):
        _=get_translator(self.language)

        layer_values_str=_("Layer values")
        input_str=_("INPUTS:")
        thickness_str= _("Thickness")
        thermal_conductivity_str=_("Thermal conductivity λ")
        gross_density_str=_("Gross density ρ")
        specific_heat_capacity_str=_("Specific heat capacity c")
        output_str= _("OUTPUTS:")

        thermal_resistance_str=_("Thermal resistance R")
        thermal_diffusivity_str=_("Thermal diffusivity α")
        thermal_effusivity_str=_("Thermal effusivity")
        return \
f"""{self.name}: 

{thickness_str} : {self.thickness:0.3f} [m]
{thermal_conductivity_str} : {self.thermal_conductivity:0.3f} [W/mK]
{gross_density_str} : {self.gross_density:0.0f} [kg/mc]
{specific_heat_capacity_str} : {self.specific_heat_capacity:0.0f} [J/kgK]
{thermal_resistance_str} : {self.thermal_resistance:0.3f} [m²K/W]
{thermal_diffusivity_str} : {self.thermal_diffusivity:0.3f} [m²/ (s*10^6)]
{thermal_effusivity_str} : {self.thermal_effusivity:0.1f}"""


class AirLayer(LayerBase):
    def __init__(self,
        name:str,
        thickness:float,
        heat_flow_direction:str="Ho",
        is_air:bool =True,   
        language:str="en"     

        ):
        """ AirLayer non-ventilated 
        Args:
            name (str): 
            thickness (float):  "d" in m thickness of layer 
            heat_flow_direction (str):  "Ho": Horizontal (example: wall)
                                        "Up": Upwards (example Roof)
                                        "Do": Downwards (example floor)
            is_air (bool, optional): Defaults to True
            language (str, optional): Defaults to "en". 
        """
        super().__init__(name,
                      thickness,
                      is_air,
                      language)

        self.heat_flow_direction= heat_flow_direction
        self.is_air=True
        

    @property
    def thermal_resistance(self):
        """R of air layer in m²K/W
        Returns:
            float: 
        """
        return get_resistance_unventilated_air_layer(
            thickness=self.thickness,
            heat_flow_direction=self.heat_flow_direction)
    

    @property
    def thermal_conductivity(self):
        "lambda W/mK"
        return self.thickness/self.thermal_resistance
    


    def get_values(self):
        _=get_translator(self.language)

        layer_values_str=_("Layer values")

        input_str=_("INPUTS:")

        thickness_str= _("Thickness")
        thermal_conductivity_str=_("Thermal conductivity λ")
        heat_flow_direction_str=_("Heat flow direction")

        output_str= _("OUTPUTS:")

        thermal_resistance_str=_("Thermal resistance R")


        return \
f"""{self.name}:

{thickness_str} : {self.thickness:0.3f} [m]
{thermal_conductivity_str} : {self.thermal_conductivity:0.3f} [W/mK]
{heat_flow_direction_str} : {self.heat_flow_direction} "Ho": Horizontal (wall),"Up": Upwards (Roof),"Do": Downwards (floor)
{thermal_resistance_str} : {self.thermal_resistance:0.3f} m²K/W"""


import numpy as np
def get_surface_resistances(heat_flow_direction:str)-> tuple:
    """Surface resistances Rsi (Internal)and Rse (external) 
        in accordance with ISO 6946:2007 in [mK/W]

    Args:
        heat_flow_direction (str): 
            "Ho": Horizontal (example: wall)
            "Up": Upwards (example Roof)
            "Do": Downwards (example floor)

    Returns:
        tuple: (Rsi, Rse) in [mK/W]
    """

    if heat_flow_direction== "Ho":
        return (0.13 , 0.04) # (Rsi, Rse)
    elif heat_flow_direction== "Up":
        return (0.10 , 0.04) #(Rsi, Rse)
    elif heat_flow_direction== "Do":
        return (0.17 , 0.04) #(Rsi, Rse)
    else:
        raise ValueError(f"""invalid string heat_flow_direction: {heat_flow_direction}
        available choices: Ho, Up ,Do
        """)


def get_resistance_unventilated_air_layer(
        heat_flow_direction:str,
        thickness:float=0
        )->float:
    """ Thermal resistance of unventilated air layers 
        with high emissivity surfaces 
        in accordance with ISO 6946:2007 in [mK/W]
    Args:
        thickness (float): in meters
        heat_flow_direction (str): 
            "Ho": Horizontal (example: wall)
            "Up": Upwards (example Roof)
            "Do": Downwards (example floor)

    Returns:
        float: Thermal Resistance in [mK/W]
    """
    thicknesses = [0,
                   5*10**-3, #0.005 m or 5 mm
                   7*10**-3,
                   10*10**-3,
                   15*10**-3,
                   25*10**-3,
                   50*10**-3,
                   100*10**-3,
                   300*10**-3]  
    up_r_values =[0,
                  0.11,
                  0.13,
                  0.15,
                  0.16,
                  0.16,
                  0.16,
                  0.16,
                  0.16] 
    ho_r_values =[0,
                  0.11,
                  0.13,
                  0.15,
                  0.17,
                  0.18,
                  0.18,
                  0.18,
                  0.18]
    do_r_values =[0,
                  0.11,
                  0.13,
                  0.15,
                  0.17,
                  0.19,
                  0.21,
                  0.22,
                  0.23]
    
    if heat_flow_direction == "Ho":
        return np.interp(thickness, thicknesses, ho_r_values)
    elif heat_flow_direction == "Up":
        return np.interp(thickness, thicknesses, up_r_values)
    elif heat_flow_direction == "Do":
        return np.interp(thickness, thicknesses, do_r_values)
    else:
        raise ValueError(f"""invalid string heat_flow_direction: {heat_flow_direction}
        available choices: Ho, Up ,Do
        """)



import unittest
import matplotlib.pyplot as plt
import json
import numpy as np
from becalib.charts import plot_sinusoidal_wave,plot_component_layers
import math



class Testplot(unittest.TestCase):

    def test_plot(self):
        
        plt_sinu= plot_sinusoidal_wave(
            max_temp= 35, # °C
            min_temp = 28, # °C
            time_period =24, # in hours
            decrement_factor=0.25,  # ammortissement
            time_shift=6,
            language="en"
              )
        

        # plt_sinu.show()
        self.assertTrue(bool(plt_sinu))


    def test_wall_layer(self):

        plt_lay= plot_component_layers(
            names=["layer 1","layer 2","layer 3"],
            thickness=[0.1,0.5,0.8],
            heat_flow_direction="Ho",
            language="fr"
        )

        # plt_lay.show()
        self.assertTrue(bool(plt_lay))





if __name__ == '__main__':
    unittest.main()


import numpy as np
import pandas as pd
from becalib.charts import plot_component_layers, plot_sinusoidal_wave
from becalib.layers import MaterialLayer
from becalib.air_resistances import get_surface_resistances
from becalib.translator import get_translator
from becalib.algos import *
import copy


class Component():
    def __init__(self,
        name: str,
        layers: list[MaterialLayer],
        heat_flow_direction:str,
        time_period: float = 24, 
        language: str= "en"
        ):
        """Summer analysis of multi layer component like wall, roof or floor

        Args:
            name (str): component name \n
            layers (list[MaterialLayer]): ordered list of material layers interior to exterior\n
            heat_flow_direction (str): \n
                                        "Ho": Horizontal (example: wall)\n
                                        "Up": Upwards (example Roof)\n
                                        "Do": Downwards (example floor)\n
            time_period (float, optional): analysis period in [h]. Defaults to 24 h.\n

        """
        self.name=name
        self.layers=layers
        self.heat_flow_direction=heat_flow_direction
        self.time_period=time_period
        self.language=language
        
        # Compute all values
        self.update()
    
    def _set_parameter_names_strings(self):
        """set all string needed to print values labels in different languages
        """
        _=get_translator(self.language)

        self.component_str=_("Component")
        self.heat_flow_direction_str=_("Heat flow direction")
        self.time_period_str = _("Time period")

        self.surface_thermal_resistance_int_str=_("Interior surface thermal resistance Rsi")
        self.surface_thermal_resistance_ext_str=_("Exterior surface thermal resistance Rse")
        self.thickness_component_str=_("Thickness")
        self.thermal_resistance_component_str=_("Resistance")
        self.thermal_transmittance_component_str= _("Transmittance")
        self.periodic_thermal_transmittance_str = _("Periodic transmittance")
        self.decrement_factor_str=_("Decrement factor")
        self.time_shift_str=_("Time shift")
        self.thermal_admittance_int_str=_("Interior admittance")
        self.thermal_admittance_ext_str=_("Exterior admittance")
        self.areal_heat_capacity_int_str=_("Interior areal heat capacity")
        self.areal_heat_capacity_ext_str=_("Exterior areal heat capacity")
        self.areal_heat_capacity_str=_("Areal heat capacity")

        self.mass_component_str= _("Surface mass")

        self.threshold_values_italian_dm_26_06_2009_str=_("Summer performance")

    def update(self):
        """Compute all values with last inputs
        """
        # Surface resistances Rsi (Internal)and Rse (external)
        (self.surface_thermal_resistance_int,
        self.surface_thermal_resistance_ext)=get_surface_resistances(
                                                heat_flow_direction=self.heat_flow_direction)
        
        self._set_parameter_names_strings()

        # Override heat_flow_direction for air layers
        list_of_layers= []
        for layer in self.layers:
            if layer.is_air is True:
                layer.heat_flow_direction = self.heat_flow_direction
                list_of_layers.append(layer)
            else:
                list_of_layers.append(layer)
        
        self.layers = list_of_layers

        # np.array of thickness of each Layer
        self.thicknesses= np.array([layer.thickness for layer in self.layers])

        # sum of thicknesses of all layers in [m]
        self.thickness_component= np.sum(self.thicknesses,)

        # array of thermal conductivities "λ" lambda [W/mK] Layer by Layer 
        self.thermal_conductivities= np.array([layer.thermal_conductivity for layer in self.layers])

        # array of gross densities "ρ" rho [kg/mc] Layer by Layer  
        densities_list = []
        for layer in self.layers:
            if layer.is_air is False:
                densities_list.append(layer.gross_density)
            else:
                densities_list.append(None)
        self.gross_densities= np.array(densities_list)

        # array of Specific heat capacities "c" [J/kgK] Layer by layer
        specific_heats_list=[]
        for layer in self.layers:
            if layer.is_air is False:
                specific_heats_list.append(layer.specific_heat_capacity)
            else:
                specific_heats_list.append(None)

        self.specific_heat_capacities= np.array(specific_heats_list)

        ##  Steady-State Thermal Analysis ##

        # thermal_resistances
        # array of all thermal resistances Layer by layer
        # including internal and external surface thermal resistances
        resistances = np.array([layer.thermal_resistance for layer in self.layers])
        resistances = np.insert(resistances, 0, self.surface_thermal_resistance_int)
        resistances = np.append(resistances, self.surface_thermal_resistance_ext)

        self.thermal_resistances= resistances

        # Component thermal resistance R in [m²K/W]
        # sum of layer resistances including internal 
        # and external resistances
        self.thermal_resistance_component = np.sum(self.thermal_resistances)


        #thermal_transmittance_component U-value in  W/m²K)
        self.thermal_transmittance_component= 1 / self.thermal_resistance_component
    
    
        ###  Dynamic Thermal Analysis ###

        # periodic_penetration_depth
        self._periodic_penetration_depth_list=get_periodic_penetration_depth_list(layers= self.layers,
                                            time_period= self.time_period
                                            )
        # XI
        self._xi_list=get_xi_list(layers=self.layers,
                                  periodic_penetration_depth_list= self._periodic_penetration_depth_list
                                  
        )
        # heat_transfer_matrix layer by layer
        self._heat_transfer_matrix_layer_list = \
            get_heat_transfer_matrix_layer_list(
                thermal_resistances=self.thermal_resistances,
                xi_list=self._xi_list,
                periodic_penetration_depth_list=self._periodic_penetration_depth_list,
                thermal_conductivities=self.thermal_conductivities)
        
        # heat_transfer_matrix component
        self._heat_transfer_matrix_component = \
            get_heat_transfer_matrix_component(
                ht_matrix_list= self._heat_transfer_matrix_layer_list,
                surface_thermal_resistance_int=self.surface_thermal_resistance_int,
                surface_thermal_resistance_ext=self.surface_thermal_resistance_ext) 
        
        # periodic_thermal_transmittance
        self.periodic_thermal_transmittance= \
            get_periodic_thermal_transmittance(heat_transfer_matrix_component=self._heat_transfer_matrix_component)
        
        # decrement_factor
        self.decrement_factor=get_decrement_factor(
            periodic_thermal_transmittance= self.periodic_thermal_transmittance,
            thermal_transmittance_component= self.thermal_transmittance_component)
        
        # time_shift
        self.time_shift=get_time_shift(
            heat_transfer_matrix_component=self._heat_transfer_matrix_component,
            time_period=self.time_period)
        
        # thermal_admittance_int
        self.thermal_admittance_int= get_thermal_admittance_int(self._heat_transfer_matrix_component) 
        # thermal_admittance_ext
        self.thermal_admittance_ext= get_thermal_admittance_ext(self._heat_transfer_matrix_component) 

        # areal_heat_capacity_int
        self.areal_heat_capacity_int= get_areal_heat_capacity_int(
            self._heat_transfer_matrix_component,
            self.time_period )        
        # areal_heat_capacity_ext        
        self.areal_heat_capacity_ext= get_areal_heat_capacity_ext(
            self._heat_transfer_matrix_component,
            self.time_period)
        # areal_heat_capacity_component
        self.areal_heat_capacity_component=\
            get_areal_heat_capacity_component(self.layers)
        
        # time_constant
        self.time_constant= get_time_constant(
            self.areal_heat_capacity_component, 
            self.thermal_resistance_component)
        
        # threshold_values_italian_dm_26_06_2009
        self.threshold_values_italian_dm_26_06_2009=\
            get_threshold_values_italian_dm_26_06_2009(
                self.time_shift,
                self.decrement_factor,
                self.language
            )
        
        # mass_component        
        self.mass_component=get_mass_component(self.layers)

    # Methods to get computed values by strings, DataFrames or charts
    def get_layers_dataframe(self,
            data_type:str="st"):
        """get pandas dataframe or styler objet (for notebooks) of layers

        Args:
            data_type (str, optional): _description_. Defaults to "st"=styler objet or "df"=dataframe

        Returns:
            object: pandas dataframe or styler
        """

        list_of_layers_dict=[]

        _=get_translator(language=self.language)

        list_of_layers_dict.append({"name":_("Interior surface"),
                                    "thermal_resistance":self.surface_thermal_resistance_int}
                                    )
        for layer in self.layers:
            layer_dict= {}

            if layer.is_air==False:
                dict_of_computed_values= {"thermal_resistance":layer.thermal_resistance,
                                   "thermal_diffusivity":layer.thermal_diffusivity,
                                   "thermal_effusivity":layer.thermal_effusivity
                                   }

                layer_dict=dict(copy.copy(layer.__dict__))


                layer_dict.pop("language")
                layer_dict.update(dict_of_computed_values)

            if layer.is_air==True:
                
                dict_of_computed_values= {"thermal_resistance":layer.thermal_resistance}
                layer_dict=dict(copy.copy(layer.__dict__))

                layer_dict.pop("heat_flow_direction")
                
                layer_dict.update(dict_of_computed_values)


            list_of_layers_dict.append(layer_dict)
        
        list_of_layers_dict.append({"name":_("Exterior surface"),
                                    "thermal_resistance":self.surface_thermal_resistance_ext}
                                    )
        
        df=pd.DataFrame(list_of_layers_dict)

        df=df.reindex(columns=[
            'name',
            'thickness',
            'thermal_conductivity',
            "thermal_resistance",
            'is_air',
            "gross_density",
            "specific_heat_capacity",
            
            "thermal_diffusivity",
            "thermal_effusivity"
        ])
    
        df = df.rename(columns=
                {'name': _('Name'),
                'thickness': _("Thickness [m]"),
                'is_air': _('is_air'),
                'thermal_conductivity': _('Conductivity λ [W/mK]'),
                "gross_density":_("Gross density ρ [kg/m³]"),
                "specific_heat_capacity":_("Specific heat capacity c [J/kgK]"),
                "thermal_resistance":_("Resistance R [m²K/W]"),
                "thermal_diffusivity":_("Diffusivity α [m²/ (s*10^6)]"),
                "thermal_effusivity":_("Effusivity")

                })
        
        style_object=df.style.format(
            {_("Thickness [m]"): '{:.2f}', 
            _('Conductivity λ [W/mK]'): '{:.3f}',
            _("Gross density ρ [kg/m³]"): '{:.0f}',
            _("Specific heat capacity c [J/kgK]"): '{:.0f}',
            _("Resistance R [m²K/W]"): '{:.3f}',
            _("Diffusivity α [m²/ (s*10^6)]"): '{:.3f}',
            _("Effusivity"): '{:.1f}',
            }, 
            na_rep='', precision=1)
        
        if data_type=="st":
            return style_object
        else:
            return df
    
    def _get_layers_data_string(self)->str:
        """get string of data layer by layer

        Returns:
            str: layers data
        """

        _=get_translator(self.language)

        #list of layer data
        out_layers_str=_("Layers:")
        out_layers_str=out_layers_str+"\n"+ "-----------------------"+"\n"+_("Interior")
        for layer in self.layers:
            out_layers_str=out_layers_str+ "\n"+"-----------------------"
            layer.language= self.language
            out_layers_str=out_layers_str+ "\n" + layer.get_values()
        out_layers_str=out_layers_str+"\n"+ "-----------------------"+"\n"+ _("Exterior")+"\n"+ "-----------------------"

        return out_layers_str

    def get_values(self):
        """get string of all inputs and output values
        """
        return \
f"""#######################################
{self.component_str}: {self.name}

{self.heat_flow_direction_str}: {self.heat_flow_direction} (|"Ho": Horizontal (wall) |"Up": Upwards (Roof) | "Do": Downwards (floor))
{self.time_period_str}: {self.time_period} [h]

{self.surface_thermal_resistance_int_str}: {self.surface_thermal_resistance_int} [m²K/W]
{self.surface_thermal_resistance_ext_str}: {self.surface_thermal_resistance_ext} [m²K/W]

{self.thickness_component_str}: {self.thickness_component:.3f} [m]
{self.thermal_resistance_component_str}: {self.thermal_resistance_component:.3f} [m²K/W] Rsi and Rse included
{self.thermal_transmittance_component_str}: {self.thermal_transmittance_component:.3f} [W/m²K]

{self.periodic_thermal_transmittance_str}: {self.periodic_thermal_transmittance:.3f} [W/m²K]
{self.decrement_factor_str}: {self.decrement_factor:.3f} [-]
{self.time_shift_str}: {self.time_shift:.1f} [h]

{self.thermal_admittance_int_str}: {self.thermal_admittance_int:.3f} [W/m²K]
{self.thermal_admittance_ext_str}: {self.thermal_admittance_ext:.3f} [W/m²K]

{self.areal_heat_capacity_int_str}: {self.areal_heat_capacity_int:.3f} [kJ/m²K] 
{self.areal_heat_capacity_ext_str}: {self.areal_heat_capacity_ext:.3f} [kJ/m²K]

{self.areal_heat_capacity_str}: {self.areal_heat_capacity_component:.3f} [kJ/m²K]

{self.threshold_values_italian_dm_26_06_2009_str}: {self.threshold_values_italian_dm_26_06_2009} (in accordance with italian DM 26/06/2009)
{self.mass_component_str}: {self.mass_component:.1f} [kg/m²]

{self._get_layers_data_string()}
#######################################"""

    def get_summer_performance_key_values(self):
        """get string of component key values inputs and output
        """
        return \
f"""#######################################
{self.component_str}: {self.name}

{self.time_period_str}: {self.time_period} [h]

{self.thickness_component_str}: {self.thickness_component:.3f} [m]
{self.thermal_resistance_component_str}: {self.thermal_resistance_component:.3f} [m²K/W] Rsi and Rse included
{self.thermal_transmittance_component_str}: {self.thermal_transmittance_component:.3f} [W/m²K]

{self.decrement_factor_str}: {self.decrement_factor:.3f} [-]
{self.time_shift_str}: {self.time_shift:.1f} [h]

{self.areal_heat_capacity_int_str}: {self.areal_heat_capacity_int:.3f} [kJ/m²K] 

{self.threshold_values_italian_dm_26_06_2009_str}: {self.threshold_values_italian_dm_26_06_2009} (in accordance with italian DM 26/06/2009)
{self.mass_component_str}: {self.mass_component:.1f} [kg/m²]

#######################################"""

    def get_component_layers_chart(self):
        """get a matplotlib pyplot object of component layers
        Returns:
            pyplot: matplotlib pyplot object
        """       
        layer_names= np.array([layer.name for layer in self.layers])     
        return plot_component_layers(
                        names=layer_names,
                        thickness=self.thicknesses,
                        heat_flow_direction=self.heat_flow_direction,
                        language=self.language
                        )
    def get_component_sinusoidal_wave_chart(self,
            max_temp:float = 35, 
            min_temp:float = 28,):
        """sinusoidal waves matplotlib pyplot object of time_shift and decrement_factor

        Args:
            max_temp (float, optional): Max temperature. Defaults to 35°C.
            min_temp (float, optional): Min temperature. Defaults to 28°C.

        Returns:
            matplotlib pyplot object: sinusoidal waves
        """

        return plot_sinusoidal_wave(
            decrement_factor= self.decrement_factor, 
            time_shift=self.time_shift,
            time_period=self.time_period,
            max_temp=max_temp, 
            min_temp=min_temp, 
            language=self.language
                )


import unittest
from becalib.air_resistances import get_surface_resistances, get_resistance_unventilated_air_layer


class TestAirResist(unittest.TestCase):

    def test_get_res_ho(self):
        self.assertEqual((0.13 , 0.04),get_surface_resistances(heat_flow_direction="Ho"))
    
    def test_get_res_up(self):
        self.assertEqual((0.10 , 0.04),get_surface_resistances(heat_flow_direction="Up"))

    def test_get_res_do(self):
        self.assertEqual((0.17 , 0.04),get_surface_resistances(heat_flow_direction="Do"))

    def test_get_resistance_unventilated_air_layer(self):

        self.assertEqual(
            get_resistance_unventilated_air_layer(
            heat_flow_direction="Do",
            thickness=5*10**-3),
            0.11
            )
        
        self.assertEqual(
            get_resistance_unventilated_air_layer(
            heat_flow_direction="Do",
            thickness=15*10**-3),
            0.17
            )
        self.assertEqual(
            get_resistance_unventilated_air_layer(
            heat_flow_direction="Up",
            thickness=10*10**-3),
            0.15
            )
        
        self.assertEqual(
            get_resistance_unventilated_air_layer(
            heat_flow_direction="Ho",
            thickness=0*10**-3),
            0
            )
        self.assertEqual(
            get_resistance_unventilated_air_layer(
            heat_flow_direction="Ho",
            thickness=2),
            0.18
            )
        



if __name__ == '__main__':
    unittest.main()


import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from becalib.translator import get_translator



def plot_sinusoidal_wave(            
    decrement_factor: float, 
    time_shift: float,
    time_period:float =24,
    max_temp:float = 35, 
    min_temp:float = 28, 
    language:str="en"
     )-> plt:
    """sinusoidal_wave matplotlib pyplot object of time_shift and decrement_factor 

    Args:
        decrement_factor (float): in [-]
        time_shift (float): in [h]
        time_period (float, optional): in [h]. Defaults to 24h.
        max_temp (float, optional): in °C. Defaults to 35°C.
        min_temp (float, optional): in °C. Defaults to 28°C.
        language (str, optional): Defaults "en", ["fr","en","it"]

    Raises:
        TypeError: all parameters cannot be None

    Returns:
        plt: matplotlib pyplot object
    """

    _=get_translator(language)

    # Do not accept None
    for k,v in locals().items():
        if v is None:
            raise TypeError(f"{k} cannot be None") 

    temp_average = (max_temp+min_temp)/2

    amplitude_ext=max_temp-temp_average
    b=2*np.pi/time_period 
    c=0

    x = np.arange(0,2*time_period,0.1) 

    # exterior sinusoidal wave
    y = amplitude_ext* np.sin(b*(x-0))+ temp_average

    amplitude_int=amplitude_ext*decrement_factor

    
    # interior sinusoidal wave

    z = amplitude_int* np.sin(b*(x-time_shift))+ temp_average

    plt.plot(x,y, x,z)
    
    # Setting title for the plot in black color

    time_shift_str=_('Time shift')
    decrement_factor_str=_("Decrement factor")

    title_str=\
f"""{time_shift_str}:  {time_shift:.1f} [h]
    {decrement_factor_str}: {decrement_factor:.2f} [-]
    """
    
    # plt.title(_('Time shift'),  fontsize=12, color='black')
    plt.title(title_str,  fontsize=10, color='black')

    # Setting x axis label for the plot
    plt.xlabel(_('Time [h]'),  fontsize=10, color='black')

    # Setting y axis label for the plot
    plt.ylabel(_('Temperature [°C]'),  fontsize=10, color='black')

    # Showing grid
    plt.grid()

    # Highlighting axis at x=0 and y=0
    plt.axhline(y=temp_average, color='k')
    plt.axvline(x=0, color='k')

    plt.xticks(np.arange(0,49,6))
    plt.yticks(np.arange(min_temp-1,max_temp+2,1))

    plt.legend([_("Text"), _('Tsurf_int')])     


    return plt



def plot_component_layers(
        names:list,
        thickness:list,
        heat_flow_direction:str,
        language:str="en"
    ):
    """get a matplotlib pyplot object of component layers

    Args:
        names (list): list of layers names
        thickness (list): list of layers thicknesses
        heat_flow_direction (str): 
                "Ho": Horizontal (example: wall)
                "Up": Upwards (example Roof)
                "Do": Downwards (example floor)


    Returns:
        plt: matplotlib pyplot object
    """

    # Do not accept None
    for k,v in locals().items():
        if v is None:
            raise TypeError(f"{k} cannot be None") 
        
    if heat_flow_direction=="Do":
        names= list(reversed(names))
        thickness= list(reversed(thickness))

    
    dict_for_df={}
    for i in range(0,len(names)):
        dict_for_df[names[i]]=thickness[i]
    df = pd.DataFrame(dict_for_df, index=[""])
    
    width = np.sum(np.array(thickness))*10

    _=get_translator(language)

    interior_string= _("Interior")
    exterior_string=_("Exterior")
    thickness_string = _("thickness [m]")


    if heat_flow_direction=="Ho":

        ax = df.plot.barh(width=width,stacked=True)
        ax2 = ax.twinx()

        ax.set_title(_("Component layers horizontal heat flow"), fontsize=12, color='black' )

        ax.set_ylabel(interior_string, fontsize=10, color='black' )
        ax2.set_ylabel(exterior_string, fontsize=10, color='black' )

        ax2.set_yticklabels([])
        ax2.set_yticks([])
        ax.set_yticks([])
        ax.set_xlabel(thickness_string, fontsize=10)


    elif heat_flow_direction=="Up":

        ax = df.plot.bar(width=width,stacked=True)
        ax2 = ax.twiny()

        ax.set_title(_("Component layers upwards heat flow"), fontsize=12, color='black' )

        ax.set_xlabel(interior_string, fontsize=10, color='black' )
        ax2.set_xlabel(exterior_string, fontsize=10, color='black' )

        ax2.set_xticklabels([])
        ax2.set_xticks([])
        ax.set_xticks([])

        ax.set_ylabel(thickness_string, fontsize=10)
    
    elif heat_flow_direction=="Do":

        ax = df.plot.bar(width=width,stacked=True)
        ax2 = ax.twiny()

        ax.set_title(_("Component layers downward heat flow"), fontsize=12, color='black' )

        ax.set_xlabel(exterior_string, fontsize=10, color='black' )
        ax2.set_xlabel(interior_string, fontsize=10, color='black' )

        ax2.set_xticklabels([])
        ax2.set_xticks([])
        ax.set_xticks([])

        ax.set_ylabel(thickness_string, fontsize=10)
    
    else:
        raise ValueError(f"""invalid string heat_flow_direction: {heat_flow_direction}
        available choices: Ho, Up ,Do
        """)

    return plt




# Import gettext module
import gettext
import os


def get_locales_abs_path():
    absolute_path = os.path.dirname(__file__)
    return  os.path.join(absolute_path, "locales") #full_path


def get_translator(language="en"):

    # Set the local directory
    appname = 'becalib'


    language_i18n = gettext.translation(appname, get_locales_abs_path(), fallback=True, languages=[language])

    language_i18n.install()
    _=language_i18n.gettext

    return _